#ifndef APERTURE_H
#define APERTURE_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include "luav.h"

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>

#include "xmatrix.h"

typedef enum { 
  NOTAPPLICABLE, 
  BLOCKED, 
  PASSED, 
  REDIRECTED, 
  GENERATE,
  MODIFIED,
  MAXPROCESSRESULT  /* This must be the last entry in this structure,
		       see the Aperture structure for details */
} ProcessResult;

typedef struct {
  unsigned long assembly;
  unsigned long aperture;
  char     *label;
} ApertureId;

typedef struct {
  ApertureId id;
  XMatrix forward;
  XMatrix backward;
  struct ApertureModule *module;
  void   *data_object;
  /*
    The following two struct members are the reason why MAXPROCESSRESULT
    must be the last item defined in the ProcessResult enum.
  */
  unsigned long status[ MAXPROCESSRESULT ];
  double weight[ MAXPROCESSRESULT ];
} Aperture;

typedef struct ApertureModule
{
  char *name;				/* name of lua function */
  lua_CFunction	function;		/* the function to be called
					   from lua */

  void (*init)(Aperture *aperture, void* udata);
  					/* the module init function */

  ProcessResult (*process)(Aperture *aperture);
  				/* the photon processing function */

  void (*cleanup)(Aperture *aperture);
  					/* the aperture cleanup function */
} ApertureModule;


#define	APERTURELABEL "label"

extern ApertureModule *modules[];

void aperture_empty( Aperture *aperture);
void aperture_free( Aperture *aperture);
void aperture_new(ApertureModule *module, void *data_object, const char *label);
int  aperture_print( Aperture *aperture );
void aperture_cp( Aperture* from, Aperture* to );
int  aperture_init( void* aperture, void* udata );


#endif	/* APERTURE_H */
