/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdlib.h>

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include "assembly.h"
#include "aperture_queue.h"

static char *assembly_name[] =
{
  "assembly", "subassembly", "unknown"
};


/*--------------------------------------------------------------------*/
void
assembly_free( Assembly *assembly )
{
  sll_delete(assembly->leaves, (SLL_DELETE) aperture_free);
  free(assembly);
}

int
assembly_init( void* assembly, void* udata )
{
  return sll_utraverse( ((Assembly*)assembly)->leaves, aperture_init, udata );
}


/*--------------------------------------------------------------------*/
void
assembly_print_type( AssemblyType type )
{
  tf_enter("print_assembly_type");

  if ( type != ASSEMBLY && type != SUBASSEMBLY )
    tf_exit( ExitERR_illegalArg, "Unknown assembly type: %d", type );
  else
    fprintf( stderr, assembly_name[type] );

  tf_leave();
}

/*--------------------------------------------------------------------*/
void
assembly_print( Assembly* assembly ) 
{
  if ( ! aperture_queue_print( assembly->leaves ) )
    tf_message("print_assembly: no modules in assembly\n");
}

const char *
assembly_get_name( AssemblyType type )
{
  return assembly_name[type == ASSEMBLY || type == SUBASSEMBLY
		       ? type
		       : ASSEMBLY_UNKNOWN
		       ];
}



/*
  PROCEDURE
  Copy an assembly node in a linklist to an array.

 */
int
assembly_cp( void *assembly, void *udata )
{
  Assembly *from = (Assembly*) assembly;
  AssemblyArray **array_pp = udata;
  AssemblyArray *to = *array_pp;

  to->id        = from->id;
  to->block     = from->block;
  to->num_apertures = sll_count( from->leaves );
  to->apertures = aperture_queue_array_new( from->leaves );
  
  (*array_pp)++;

  return 0;
}


