#include <stdio.h>

#include "luav.h"

#include "aperture.h"

const char*
get_opt_label( 
	      lua_State *L,
	      int narg
	       )
{
    const char* label = NULL;

    if ( lua_gettop( L ) > narg )
    {
	luaL_checktype( L, ++narg, LUA_TTABLE );
	lua_getfield( L, narg, APERTURELABEL );

	if ( lua_isstring( L, narg ) )
	{
	    label = lua_tostring( L, narg );
	}
	else if ( ! lua_isnil( L, narg ) )
	{
	    luaL_error( L, "value of %s option must be a string", APERTURELABEL );
	}
    }

    return label;
}


void stackDump( lua_State *L )
{
    int i;
    int top = lua_gettop(L);

    for ( i = 1; i <= top ; i++ )
    {
	int t = lua_type(L, i );
	switch( t )
	{
	case LUA_TSTRING:
	    fprintf( stderr, "%d: string: '%s'\n", i, lua_tostring(L, i ));
	    break;

	case LUA_TBOOLEAN:
	    fprintf( stderr, "%d: boolean: %s\n", i, lua_toboolean(L,i) ? "true" : "false");
	    break;

	case LUA_TNUMBER:
	    fprintf( stderr, "%d: number: %g\n", i, lua_tonumber(L, i));
	    break;

	default:
	    fprintf( stderr, "%d: type: %s\n", i, lua_typename(L,t));
	    break;
	}
    }
    fprintf( stderr, "\n");
}
