/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#include <stdio.h>
#include <libgen.h>

#include "luav.h"

#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <suplib/times.h>

#include "config.h"
#include "params.h"

#include "init.h"
#include "process.h"

#include "lua_iface.h"
#include "usage.h"

int 
main( int argc, char **argv )
{
  lua_State *L;
  Params io;

  argv[0] = basename(argv[0]);

  tf_init(argv[0], 0, 0);

  get_parameters( argc, argv, &io );

  if ( io.version ) {
    printf("%s %s\n", argv[0], PACKAGE_VERSION );
    return EXIT_SUCCESS;
  }

  if ( io.help ) {
    usage( );
    return EXIT_SUCCESS;
  }

  initialize_xstack();
  initialize_assembly_queue();

  L = register_lua( &io );
  exec_lua( &io, L );
  
#ifdef TEST
  startTime(NULL);
#endif
  process_all_assemblies( &io );
#ifdef TEST
  tf_message("%s\n", stringTime(elapsedTime(NULL)));
#endif

  cleanup_assembly_queue( );
  cleanup_xstack();

  return EXIT_SUCCESS;
}
