#ifndef RAY_TRANSFORMS_H
#define RAY_TRANSFORMS_H

/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2006 Smithsonian Astrophysical Observatory
 *
 * This file is part of aperture
 *
 * aperture is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * aperture is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the 
 *       Free Software Foundation, Inc. 
 *       51 Franklin Street, Fifth Floor
 *       Boston, MA  02110-1301, USA
 *
 * -->8-->8-->8-->8-- */

#define PROJECT_RAY(position,direction,z_pos)		\
  do {							\
    double _z = (z_pos);				\
    DVector3 *_p = (position);				\
    DVector3 *_d = (direction);				\
    double _t = _z - _p->z / _d->z;			\
    _p->x += _t * _d->x;				\
    _p->y += _t * _d->y;				\
    _p->z = _z;						\
  } while (0)

#define TRANSFORM_POSITION(matrix,vec_in,vec_out)				\
  do										\
  {										\
    double **_tran = (matrix);							\
    DVector3 *_in = (vec_in);							\
    DVector3 *_out = (vec_out);							\
    _out->x = _in->x * _tran[0][0] + _in->y * _tran[1][0] + _in->z * _tran[2][0] + _tran[3][0]; \
    _out->y = _in->x * _tran[0][1] + _in->y * _tran[1][1] + _in->z * _tran[2][1] + _tran[3][1]; \
    _out->z = _in->x * _tran[0][2] + _in->y * _tran[1][2] + _in->z * _tran[2][2] + _tran[3][2];	\
  } while(0)

#define TRANSFORM_DIRECTION(matrix,vec_in,vec_out)			 		\
  do											\
  {											\
    double **_tran = (matrix);								\
    DVector3 *_in = (vec_in);								\
    DVector3 *_out = (vec_out);								\
    _out->x = _in->x * _tran[0][0] + _in->y * _tran[1][0] + _in->z * _tran[2][0];	\
    _out->y = _in->x * _tran[0][1] + _in->y * _tran[1][1] + _in->z * _tran[2][1];	\
    _out->z = _in->x * _tran[0][2] + _in->y * _tran[1][2] + _in->z * _tran[2][2];	\
} while(0)


#endif	/* RAY_TRANSFORMS_H */
