#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include <bpipe/bpipe.h>
#include <bpipe/datatypes.h>

/* simple print error and exit routine */
#define error(string)                                                   \
  do {                                                                  \
      fprintf( stderr, __FILE__ " %d: %s\n", __LINE__, string );        \
      exit( EXIT_FAILURE );                                             \
     } while (0)

/*
  invoke as

  create
  create nphotons
  create x y z
  create x y z dx dy dz

*/



int 
main (int argc, char *argv[])
{
  BPipe *bpipe;
  BPipeOutput *bpo;
  double *weight;
  DVector3 *position;
  DVector3 *direction;
  unsigned int *id;

  void *data;

  size_t i;
  size_t nphots = 20;
  


  if ( NULL == ( bpipe = bpipe_new( ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* create new data packet fields */

  if ( bpipe_dpktf_add( bpipe,  "weight",     BPDType_double,   NULL ) ||
       bpipe_dpktf_add( bpipe,  "position",   BPDType_DVector3, NULL ) ||
       bpipe_dpktf_add( bpipe,  "direction",  BPDType_DVector3, NULL ) ||
       bpipe_dpktf_add( bpipe,  "id",         BPDType_uint,     NULL )
       )
    error( bpipe_strerror( bpipe_errno ) );

  if ( NULL == ( bpo = bpipe_output( bpipe, "stdout" ) ) )
    error( bpipe_strerror( bpipe_errno ) );
    
  /* map data packet fields */
  if ( NULL == ( data = bpipe_map_alloc( bpipe, 1, NULL ) ) )
    error( bpipe_strerror( bpipe_errno ) );

  /* output header */
  if ( bpipe_write_hdr(bpipe) )
    error( bpipe_strerror( bpipe_errno ) );

  /* get handles to data packet fields */
  weight    = bpipe_dpktf_datap( bpipe, data, "weight",    BPDType_double   );
  position  = bpipe_dpktf_datap( bpipe, data, "position",  BPDType_DVector3 );
  direction = bpipe_dpktf_datap( bpipe, data, "direction", BPDType_DVector3 );
  id        = bpipe_dpktf_datap( bpipe, data, "id",        BPDType_uint     );

  *id = 0;
  *weight = 1;

#define WRITE_PHOTON(_x,_y,_z,_dx,_dy,_dz)			\
  do {							\
    double norm;					\
							\
    (*id)++;						\
							\
    position->x = _x;					\
    position->y = _y;					\
    position->z = _z;					\
							\
    direction->x = _dx;					\
    direction->y = _dy;					\
    direction->z = _dz;					\
							\
    norm = sqrt(    direction->x*direction->x		\
		  + direction->y*direction->y		\
		  + direction->z*direction->z );	\
							\
    direction->x /= norm;				\
    direction->y /= norm;				\
    direction->z /= norm;				\
							\
    if ( bpipe_write_dpkt( bpipe, data, bpo ) )		\
      error( "error writing photon" );			\
							\
  } while(0)


  if ( argc-1 <= 1 )
  {
    nphots = argc == 2 ? atoi( argv[1] ) : 20;

    for (i = 1; i <= nphots ; i++)
      WRITE_PHOTON( i, i+1, i+2, sqrt(i), i*i, i*i*i );
  }


  else if ( argc-1 == 3 )
  {
    WRITE_PHOTON( atof(argv[1]), atof(argv[2]), atof(argv[3]),
		0, 0, 1 );
  }

  else if ( argc-1 == 6 )
  {
    WRITE_PHOTON( atof(argv[1]), atof(argv[2]), atof(argv[3]),
		  atof(argv[4]), atof(argv[5]), atof(argv[6]) );
  }


  bpipe_delete(bpipe);

  return EXIT_SUCCESS;
}

